	.text
	.h8300s
h8300s_cbranch:
	bra h8300s_cbranch:8
	bt h8300s_cbranch:8
	brn h8300s_cbranch:8
	bf h8300s_cbranch:8
	bhi h8300s_cbranch:8
	bls h8300s_cbranch:8
	bcc h8300s_cbranch:8
	bhs h8300s_cbranch:8
	bcs h8300s_cbranch:8
	blo h8300s_cbranch:8
	bne h8300s_cbranch:8
	beq h8300s_cbranch:8
	bvc h8300s_cbranch:8
	bvs h8300s_cbranch:8
	bpl h8300s_cbranch:8
	bmi h8300s_cbranch:8
	bge h8300s_cbranch:8
	blt h8300s_cbranch:8
	bgt h8300s_cbranch:8
	ble h8300s_cbranch:8
	bra h8300s_cbranch:16
	bt h8300s_cbranch:16
	brn h8300s_cbranch:16
	bf h8300s_cbranch:16
	bhi h8300s_cbranch:16
	bls h8300s_cbranch:16
	bcc h8300s_cbranch:16
	bhs h8300s_cbranch:16
	bcs h8300s_cbranch:16
	blo h8300s_cbranch:16
	bne h8300s_cbranch:16
	beq h8300s_cbranch:16
	bvc h8300s_cbranch:16
	bvs h8300s_cbranch:16
	bpl h8300s_cbranch:16
	bmi h8300s_cbranch:16
	bge h8300s_cbranch:16
	blt h8300s_cbranch:16
	bgt h8300s_cbranch:16
	ble h8300s_cbranch:16

