#name: i386 relax
#objdump: -s

.*: +file format .*i386.*

Contents of section .text:
Contents of section .data:
Contents of section .gcc_except_table:
 0+000 02[ 	]*.[ 	]*
Contents of section .gnu.linkonce.t.blah:
 0+000 eb00[ 	]+..[ 	]*
