;; Test that all predefined symbol names for control registers
;; are properly accepted and translated to numeric values.  Also
;; verifies that they are diassembled correctly as symbolics.

	rdcr	ANASTAT,r2
	rdcr	BRK1,r2
	rdcr	BRK2,r2
	rdcr	CONFIG,r2
	rdcr	DLRU,r2
	rdcr	DTAG0,r2
	rdcr	DTAG1,r2
	rdcr	DTAG10,r2
	rdcr	DTAG11,r2
	rdcr	DTAG12,r2
	rdcr	DTAG13,r2
	rdcr	DTAG14,r2
	rdcr	DTAG15,r2
	rdcr	DTAG2,r2
	rdcr	DTAG3,r2
	rdcr	DTAG4,r2
	rdcr	DTAG5,r2
	rdcr	DTAG6,r2
	rdcr	DTAG7,r2
	rdcr	DTAG8,r2
	rdcr	DTAG9,r2
	rdcr	ECOMCNTL,r2
	rdcr	EIP,r2
	rdcr	EPC,r2
	rdcr	FLTADR,r2
	rdcr	FLTDTH,r2
	rdcr	FLTDTL,r2
	rdcr	FLTOP,r2
	rdcr	FLTTAG,r2
	rdcr	FPST,r2
	rdcr	IE,r2
	rdcr	ILRU,r2
	rdcr	IN0P,r2
	rdcr	IN1P,r2
	rdcr	INTPEN,r2
	rdcr	ITAG0,r2
	rdcr	ITAG1,r2
	rdcr	ITAG10,r2
	rdcr	ITAG11,r2
	rdcr	ITAG12,r2
	rdcr	ITAG13,r2
	rdcr	ITAG14,r2
	rdcr	ITAG15,r2
	rdcr	ITAG2,r2
	rdcr	ITAG3,r2
	rdcr	ITAG4,r2
	rdcr	ITAG5,r2
	rdcr	ITAG6,r2
	rdcr	ITAG7,r2
	rdcr	ITAG8,r2
	rdcr	ITAG9,r2
	rdcr	MIP,r2
	rdcr	MPC,r2
	rdcr	OUTP,r2
	rdcr	PKTREQ,r2
	rdcr	PPERROR,r2
	rdcr	SYSSTK,r2
	rdcr	SYSTMP,r2
	rdcr	TCOUNT,r2
	rdcr	TSCALE,r2
