SCRIPT_NAME=mips
OUTPUT_FORMAT="ecoff-bigmips"
BIG_OUTPUT_FORMAT="ecoff-bigmips"
LITTLE_OUTPUT_FORMAT="ecoff-littlemips"
TARGET_PAGE_SIZE=0x1000000
ARCH=mips
ENTRY=start
TEXT_START_ADDR=0xa0012000
DATA_ADDR=.
TEMPLATE_NAME=mipsecoff
EMBEDDED=yes
