	.AIF	1 EQ 1
	OK
	.AELSE
	BAD
	.AENDI
	.AIF	1 EQ 2
	BAD
	.AELSE
	OK
	.AENDI
	.AIF	1 EQ 2
	BAD
	.AELSE
	OK
	.AIF	1 EQ 2
	BAD
	.AELSE
	OK
	.AENDI
	.AENDI
	.AIF	1 LT  2
	OK
	.AENDI
	.AIF	1 EQ  2
	BAD
	.AENDI
	.AIF	1 NE  2
	OK
	.AENDI
	.AIF	1 LE  2
	OK
	.AENDI
	.AIF	1 GT  2
	BAD
	.AENDI
	.AIF	3 GE  2
	OK
	.AENDI
	.AIF	3 LT  2
	BAD
	.AENDI
	.AIF	3 EQ  2
	BAD 
	.AENDI
	.AIF	3 NE  2
	OK
	.AENDI
	.AIF	3 LE  2
	BAD
	.AENDI
	.AIF	3 GT  2
	OK
	.AENDI
	.AIF	3 GE  2
	OK
	.AENDI
	.AIF	"FOO" EQ "BAR"
	BAD
	.AENDI
	.AIF	"FOO" EQ "FOO"
	OK
	.AENDI
	.AIF	"FOO" NE "BAR"
	OK
	.AENDI
	.AIF	"FOO" EQ "FOO"
	OK
	.AENDI
	.AIF	1 EQ 1
	.AIF	1 EQ 1
	OK
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	OK
	.AENDI
	OK
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	OK
	.AENDI
	.AIF	1 EQ 1
	OK
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	.AIF	1 EQ 1
	OK
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	OK
	.AENDI
	OK
	.AENDI
	.AIF	1 EQ 1
	OK
	.AIF	1 EQ 1
	OK
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	OK
	.AENDI
	.AELSE	
	BAD
	.AENDI
	.AIF	1 EQ 0
	BAD
	.AELSE	
	OK
	.AENDI
	.END
