/* sym_ids.h

   Copyright 2000, 2001 Free Software Foundation, Inc.

This file is part of GNU Binutils.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

#ifndef sym_ids_h
#define sym_ids_h

#include "symtab.h"

typedef enum
  {
    INCL_GRAPH = 0, EXCL_GRAPH,
    INCL_ARCS, EXCL_ARCS,
    INCL_FLAT, EXCL_FLAT,
    INCL_TIME, EXCL_TIME,
    INCL_ANNO, EXCL_ANNO,
    INCL_EXEC, EXCL_EXEC,
    NUM_TABLES
  }
Table_Id;

extern Sym_Table syms[NUM_TABLES];

extern void sym_id_add             PARAMS ((const char *, Table_Id));
extern void sym_id_parse           PARAMS ((void));
extern bool sym_id_arc_is_present  PARAMS ((Sym_Table *, Sym *, Sym *));

#endif /* sym_ids_h */
